/*
GLDM - Graphical Liberty Display Manager

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software Foundation,
Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/


#include "sdlinterface.h"

SDLInterface::SDLInterface()
{
    SDL_Init(SDL_INIT_VIDEO);
    mPerformQuit=false;
}

SDLInterface::~SDLInterface()
{
    SDL_Quit();
}

/** It is recommended that you, if you derive your interface from SDLInterface,
  * call this function. (Use: <CODE>SDLInterface::showSelf();</CODE> from within
  * your implementation of showSelf(). Or, simply do not implement showSelf();
  * the choice is yours.)
  */
void SDLInterface::showSelf()
{
    int flags, w=0, h=0, bpp=0;

    if (!SDL_WasInit(SDL_INIT_VIDEO))
        SDL_InitSubSystem(SDL_INIT_VIDEO);

    flags = SDL_SWSURFACE | SDL_DOUBLEBUF;
    #ifndef THEMEDEBUG
    flags |= SDL_ANYFORMAT | SDL_FULLSCREEN;
    #else
    w=800; h=600;
    #endif

    mScreen = SDL_SetVideoMode(w, h, bpp, flags);

    if (!mScreen)
    {
        printf("Video initialization has failed!\n");
        mPerformQuit=true;
    }
}

/** If you implement hideSelf(), it is still recommended that you call this
  * function from within. (Use: <CODE>SDLInterface::hideSelf();</CODE> from within
  * your implementation of hideSelf(). Or, simply do not implement hideSelf();
  * the choice is yours.)
  */
void SDLInterface::hideSelf()
{
    SDL_QuitSubSystem(SDL_INIT_VIDEO);
}

/** If you're using some kind of switch in your SDLInterface-based UI, please
  * call SDLInterface::handleEvent(k, event) in your "default" section, to
  * allow default event handler to happen sometime. In fact, if you don't
  * handle the event, please pass it through. Thank you!
  */
void SDLInterface::updateSelf(double k)
{
    SDL_Event event;
    while(SDL_PollEvent(&event)){
        handleEvent(k, event);
    }
}

/** Default event handler for SDL UIs.
  */
void SDLInterface::handleEvent(double k, const SDL_Event& event)
{
    switch (event.type){
        case SDL_VIDEORESIZE:
            //g_engine->doResize(event.resize.w, event.resize.h);
            break;

        case SDL_QUIT:
            mPerformQuit=true;
            break;

        case SDL_KEYUP:

            break;

        case SDL_KEYDOWN:

            break;

        case SDL_MOUSEBUTTONUP:
        case SDL_MOUSEBUTTONDOWN:

            break;

        case SDL_MOUSEMOTION:

            break;
        default:
            break;
    }
}
/** This is not much smarter than built-in SDL function, but takes less
  * args. Notably it does not reload the image from cache in case the
  * the SDL driver requires it (e.g. directx could sometimes unload the
  * graphics and we'd need to refill).
  *
  * This is temporary convenience function until a proper SDL surface
  * class is written.
  **/
void SDLInterface::blitSurface(SDL_Surface*s, int dx, int dy, int sx, int sy, int w, int h)
{
    if (w==0) w=s->w;
    if (h==0) h=s->h;

    SDL_Rect src = {(int)sx,(int)sy,(int)(w),(int)(h)};
	SDL_Rect dst = {(int)dx,(int)dy,(int)(w),(int)(h)};
	while(SDL_BlitSurface(s, &src, mScreen, &dst) == -2)
	{
        while(SDL_LockSurface(s) < 0 )
        {
            SDL_Delay(10);
		}
		//loadSurfaceFromFile(filename);

		SDL_UnlockSurface(s);
	}
}
